/*
    Copyright  1995-2001, The AROS Development Team. All rights reserved.
    $Id: closecatalog.c,v 1.5 2015/08/10 13:42:08 itix Exp $

    Desc: Close a message catalog.
    Lang: english
*/
#include <exec/types.h>
#include <proto/exec.h>
#include <proto/hashtable.h>
#include "locale_intern.h"

#define DEBUG_CLOSECATALOG(x)  ;

/*****************************************************************************

    NAME */
#include <proto/locale.h>

	AROS_LH1(void, CloseCatalog,

/*  SYNOPSIS */
	AROS_LHA(struct Catalog *, catalog, A0),

/*  LOCATION */
	struct LocaleBase *, LocaleBase, 6, Locale)

/* ---- locale.library/CloseCatalog
 *
 *   NAME
 *        CloseCatalog -- Close a message catalog.
 *
 *   SYNOPSIS
 *        CloseCatalog(catalog);
 *                     A0
 *
 *        void CloseCatalog(struct Catalog *);
 *
 *   FUNCTION
 *        Conclude access to a message catalog, and decrement the use count.
 *        If this use count is 0, the catalog can be expunged when the
 *        system memory is running low.
 *
 *   INPUTS
 *        catalog - the message catalog to close, note that NULL is
 *                  a valid catalog.
 *
 *   RESULT
 *        The catalog is closed, and should no longer be used by the
 *        application.
 *
 *   NOTES
 *
 *   EXAMPLE
 *
 *   BUGS
 *
 *   SEE ALSO
 *        GetCatalogStr(), OpenCatalog() 
 *
 * ----
 *
 *   INTERNALS
 *
 *   HISTORY
 *   27-11-96    digulla automatically created from
 *               locale_lib.fd and clib/locale_protos.h
 */
{
	AROS_LIBFUNC_INIT
	AROS_LIBBASE_EXT_DECL(struct Library *,LocaleBase)
	struct IntCatalog *cat = (struct IntCatalog *)catalog;

	DEBUG_CLOSECATALOG(dprintf("CloseCatalog: catalog 0x%lx\n",catalog));

	if (cat != NULL)
	{
		struct IntCatalog *tmp = cat->ic_RootCatalog;

		if (cat->ic_Strings == NULL)
		{
			DeleteHashTable(cat->ic_HashTable);
			FreeMem(cat, sizeof(*cat));

			cat = tmp;
		}

		if (cat)
		{
			ObtainSemaphore (&IntLB(LocaleBase)->lb_CatalogLock);

			DEBUG_CLOSECATALOG(dprintf("CloseCatalog: catalog 0x%lx use counter: %ld\n", cat, cat->ic_UseCount));

			/* Decrement the use counter. 
			 */
			cat->ic_UseCount--;

			if (0 == cat->ic_UseCount)
			{
				REMOVE(&cat->ic_Catalog.cat_Link);

				DEBUG_CLOSECATALOG(dprintf("CloseCatalog: catalog 0x%lx disposed\n", cat));

				dispose_catalog(cat, LocaleBase);
				FreeVec(cat);
			}

			ReleaseSemaphore(&IntLB(LocaleBase)->lb_CatalogLock);
		}
	}

	DEBUG_CLOSECATALOG(dprintf("CloseCatalog: done\n"));
	AROS_LIBFUNC_EXIT
    
} /* CloseCatalog */
